from setuptools import setup

setup(name='RockPaperScissorsGame',
      version='0.1',
      description='The classic game of Rock Paper Scissors',
      packages=['RockPaperScissorsGame'],
      author='David Mixer',
      zip_safe=False)